"""
Tests for data processor module
"""

from unittest.mock import Mock, patch

from msx_serial.core.data_processor import DataBuffer, DataProcessor
from msx_serial.protocol.msx_detector import MSXProtocolDetector


class TestDataBuffer:
    """Test DataBuffer class"""

    def setup_method(self):
        """Setup test"""
        self.buffer = DataBuffer()

    def test_init(self):
        """Test initialization"""
        assert self.buffer.buffer == ""
        assert self.buffer.last_update_time == 0.0

    def test_add_data(self):
        """Test adding data to buffer"""
        with patch("time.time", return_value=123.456):
            self.buffer.add_data("test data")
            assert self.buffer.buffer == "test data"
            assert self.buffer.last_update_time == 123.456

    def test_add_multiple_data(self):
        """Test adding multiple data chunks"""
        with patch("time.time", return_value=123.456):
            self.buffer.add_data("first")
            self.buffer.add_data(" second")
            assert self.buffer.buffer == "first second"

    def test_clear(self):
        """Test clearing buffer"""
        self.buffer.add_data("test")
        self.buffer.clear()
        assert self.buffer.buffer == ""

    def test_get_content(self):
        """Test getting buffer content"""
        self.buffer.add_data("content")
        assert self.buffer.get_content() == "content"

    def test_has_content_true(self):
        """Test has_content when buffer has content"""
        self.buffer.add_data("content")
        assert self.buffer.has_content() is True

    def test_has_content_false_empty(self):
        """Test has_content when buffer is empty"""
        assert self.buffer.has_content() is False

    def test_has_content_false_whitespace(self):
        """Test has_content when buffer has only whitespace"""
        self.buffer.add_data("   \n\t  ")
        assert self.buffer.has_content() is False

    def test_is_timeout_true(self):
        """Test timeout detection when timed out"""
        with patch("time.time", side_effect=[100.0, 101.5]):
            self.buffer.add_data("test")
            result = self.buffer.is_timeout(1.0)
            assert result is True

    def test_is_timeout_false(self):
        """Test timeout detection when not timed out"""
        with patch("time.time", side_effect=[100.0, 100.5]):
            self.buffer.add_data("test")
            result = self.buffer.is_timeout(1.0)
            assert result is False


class TestDataProcessor:
    """Test DataProcessor class"""

    def setup_method(self):
        """Setup for each test"""
        self.mock_detector = Mock(spec=MSXProtocolDetector)
        self.mock_detector.unified_prompt_pattern = Mock()
        self.mock_detector.unified_prompt_pattern.search = Mock(return_value=None)
        self.processor = DataProcessor(self.mock_detector)

    def test_init(self):
        """Test initialization"""
        assert self.processor.detector == self.mock_detector
        assert isinstance(self.processor.buffer, DataBuffer)
        assert self.processor.instant_mode is False
        assert self.processor.last_sent_command is None
        assert self.processor.echo_suppressed is False
        assert self.processor.output_buffer == ""
        assert self.processor.last_prompt_content == ""

    def test_init_with_instant_mode(self):
        """Test initialization with instant mode"""
        processor = DataProcessor(self.mock_detector, instant_mode=True)
        assert processor.instant_mode is True

    def test_set_instant_mode(self):
        """Test setting instant mode"""
        assert self.processor.instant_mode is False
        self.processor.set_instant_mode(True)
        assert self.processor.instant_mode is True
        self.processor.set_instant_mode(False)
        assert self.processor.instant_mode is False

    def test_set_last_command(self):
        """Test setting last command"""
        self.processor.set_last_command("  LIST  ")
        assert self.processor.last_sent_command == "LIST"
        assert self.processor.echo_suppressed is False

    def test_set_last_command_reset_echo_suppression(self):
        """Test that setting last command resets echo suppression"""
        self.processor.echo_suppressed = True
        self.processor.set_last_command("NEW")
        assert self.processor.echo_suppressed is False

    def test_process_data_no_prompt(self):
        """Test processing data without prompt"""
        self.mock_detector.detect_prompt.return_value = False

        result = self.processor.process_data("test data")

        assert result == []
        assert self.processor.buffer.get_content() == "test data"

    def test_process_data_with_prompt(self):
        """Test processing data with prompt"""
        self.mock_detector.detect_prompt.return_value = True

        with patch.object(self.processor, "_split_prompt_data") as mock_split:
            mock_split.return_value = [("line1", False), ("A>", True)]

            result = self.processor.process_data("line1\nA>")

            assert result == [("line1", False), ("A>", True)]
            assert self.processor.buffer.get_content() == ""
            mock_split.assert_called_once()

    def test_process_data_instant_mode(self):
        """Test processing data in instant mode"""
        self.processor.set_instant_mode(True)
        self.mock_detector.detect_prompt.return_value = False

        result = self.processor.process_data("test data")

        assert result == [("test data", False)]
        assert self.processor.buffer.get_content() == "test data"

    def test_process_data_instant_mode_with_prompt(self):
        """Test processing data in instant mode with prompt"""
        self.processor.set_instant_mode(True)
        self.mock_detector.detect_prompt.return_value = True

        result = self.processor.process_data("A>")

        assert result == [("A>", False), ("", True)]
        assert self.processor.buffer.get_content() == ""
        assert self.processor.last_prompt_content == "A>"

    def test_process_data_instant_mode_with_echo_suppression(self):
        """Test processing data in instant mode with echo suppression"""
        self.processor.set_instant_mode(True)
        self.processor.set_last_command("LIST")
        self.processor.echo_suppressed = True
        self.mock_detector.detect_prompt.return_value = False

        result = self.processor.process_data("some output")

        assert result == [("some output", False)]

    def test_debug_received_data_with_debug_mode(self):
        """Test debug output when debug mode is enabled"""
        self.mock_detector.debug_mode = True

        with patch("sys.stderr") as mock_stderr:
            self.processor._debug_received_data("test data")

        # Verify that print was called with the expected debug message
        # Check if print was called (via the mock)
        assert mock_stderr.write.called or hasattr(mock_stderr, "write")

    def test_debug_received_data_without_debug_mode(self):
        """Test debug output when debug mode is disabled"""
        # Don't set debug_mode attribute

        with patch("sys.stderr") as mock_stderr:
            self.processor._debug_received_data("test data")

        # Should not call print when debug_mode is not set
        assert not mock_stderr.write.called

    def test_should_suppress_echo_true(self):
        """Test echo suppression detection - should suppress"""
        self.processor.set_last_command("LIST")

        result = self.processor._should_suppress_echo("LIST\nProgram output")
        assert result is True

    def test_should_suppress_echo_false_no_command(self):
        """Test echo suppression detection - no last command"""
        result = self.processor._should_suppress_echo("LIST\nProgram output")
        assert not result

    def test_should_suppress_echo_false_already_suppressed(self):
        """Test echo suppression detection - already suppressed"""
        self.processor.set_last_command("LIST")
        self.processor.echo_suppressed = True

        result = self.processor._should_suppress_echo("LIST\nProgram output")
        assert result is False

    def test_should_suppress_echo_false_command_not_in_content(self):
        """Test echo suppression detection - command not in content"""
        self.processor.set_last_command("LIST")

        result = self.processor._should_suppress_echo("Some other output")
        assert result is False

    def test_process_echo_suppression_with_remaining_content(self):
        """Test echo suppression processing with remaining content"""
        self.processor.set_last_command("LIST")
        self.processor.buffer.add_data("LIST\r\n  Program output")

        self.processor._process_echo_suppression("LIST\r\n  Program output")

        assert self.processor.echo_suppressed is True
        assert self.processor.buffer.get_content() == "Program output"

    def test_process_echo_suppression_no_remaining_content(self):
        """Test echo suppression processing without remaining content"""
        self.processor.set_last_command("LIST")
        self.processor.buffer.add_data("LIST")

        self.processor._process_echo_suppression("LIST")

        assert self.processor.echo_suppressed is True
        assert self.processor.buffer.get_content() == ""

    def test_process_echo_suppression_empty_remaining(self):
        """Test echo suppression processing with empty remaining content"""
        self.processor.set_last_command("RUN")
        self.processor.buffer.add_data("RUN\r\n\r\n")

        self.processor._process_echo_suppression("RUN\r\n\r\n")

        assert self.processor.echo_suppressed is True
        assert self.processor.buffer.get_content() == ""

    def test_is_likely_prompt_drive_prompt(self):
        """Test prompt pattern detection for drive prompts"""
        # Mock the unified pattern to return True for drive prompts
        self.mock_detector.unified_prompt_pattern.search.return_value = True
        assert self.processor._is_likely_prompt("A>") is True
        assert self.processor._is_likely_prompt("C:>") is True
        assert self.processor._is_likely_prompt("H>") is True

    def test_is_likely_prompt_basic_ok(self):
        """Test prompt pattern detection for BASIC Ok"""
        # Mock the unified pattern to return True for Ok
        self.mock_detector.unified_prompt_pattern.search.return_value = True
        assert self.processor._is_likely_prompt("Ok") is True

    def test_is_likely_prompt_basic_ok_without_keywords(self):
        """Test prompt pattern detection for Ok without BASIC keywords"""
        # Mock the unified pattern to return True for Ok
        self.mock_detector.unified_prompt_pattern.search.return_value = True
        assert self.processor._is_likely_prompt("Ok") is True

    def test_is_likely_prompt_basic_ok_with_keywords(self):
        """Test prompt pattern detection for Ok with BASIC keywords in content"""
        # Mock the unified pattern to return True for Ok
        self.mock_detector.unified_prompt_pattern.search.return_value = True
        content = "Microsoft BASIC Version 2.0\nOk"
        assert self.processor._is_likely_prompt(content) is True

    def test_is_likely_prompt_basic_ok_without_keywords_in_content(self):
        """Test prompt pattern detection for content ending with Ok but no BASIC keywords"""
        # Mock the unified pattern to return True for Ok
        self.mock_detector.unified_prompt_pattern.search.return_value = True
        content = "Some random text\nOk"
        assert self.processor._is_likely_prompt(content) is True

    def test_is_likely_prompt_false(self):
        """Test prompt pattern detection for non-prompts"""
        # Mock the unified pattern to return False for non-prompts
        self.mock_detector.unified_prompt_pattern.search.return_value = False
        assert self.processor._is_likely_prompt("not a prompt") is False
        assert self.processor._is_likely_prompt("A") is False
        assert self.processor._is_likely_prompt("") is False

    def test_get_prompt_patterns(self):
        """Test getting prompt patterns"""
        patterns = self.processor._get_prompt_patterns()

        # A-Zの全ドライブ（通常とコロン付き）に対応
        expected_patterns = []
        for drive in "ABCDEFGHIJKLMNOPQRSTUVWXYZ":
            expected_patterns.append(f"{drive}>")
        for drive in "ABCDEFGHIJKLMNOPQRSTUVWXYZ":
            expected_patterns.append(f"{drive}:>")
        expected_patterns.extend(["Ok", "Ready", "?Redo from start"])

        assert patterns == expected_patterns

    def test_has_basic_keywords_true(self):
        """Test BASIC keyword detection - has keywords"""
        content = "Microsoft BASIC Version 2.0\nCopyright (C) 1982"
        assert self.processor._has_basic_keywords(content) is True

    def test_has_basic_keywords_true_case_insensitive(self):
        """Test BASIC keyword detection - case insensitive"""
        content = "microsoft basic version 2.0"
        assert self.processor._has_basic_keywords(content) is True

    def test_has_basic_keywords_false(self):
        """Test BASIC keyword detection - no keywords"""
        content = "Some random text without keywords"
        assert self.processor._has_basic_keywords(content) is False

    def test_check_timeout_instant_mode_with_prompt(self):
        """Test timeout check in instant mode with prompt"""
        self.processor.set_instant_mode(True)
        self.processor.buffer.add_data("A>")
        self.mock_detector.detect_prompt.return_value = True

        with (
            patch.object(self.processor.buffer, "is_timeout", return_value=True),
            patch.object(self.processor.buffer, "has_content", return_value=True),
        ):
            result = self.processor.check_timeout(0.1)

            assert result == ("", True)
            assert self.processor.buffer.get_content() == ""
            assert self.processor.last_prompt_content == "A>"

    def test_check_timeout_instant_mode_no_prompt(self):
        """Test timeout check in instant mode without prompt"""
        self.processor.set_instant_mode(True)
        self.processor.buffer.add_data("some text")
        self.mock_detector.detect_prompt.return_value = False

        with (
            patch.object(self.processor.buffer, "is_timeout", return_value=True),
            patch.object(self.processor.buffer, "has_content", return_value=True),
        ):
            result = self.processor.check_timeout(0.1)

            assert result is None
            assert self.processor.buffer.get_content() == ""

    def test_check_timeout_no_timeout(self):
        """Test timeout check when no timeout"""
        self.processor.buffer.add_data("test")

        with patch.object(self.processor.buffer, "is_timeout", return_value=False):
            result = self.processor.check_timeout(0.1)
            assert result is None

    def test_check_timeout_with_timeout(self):
        """Test timeout check when timeout occurred"""
        self.processor.buffer.add_data("test")
        self.mock_detector.detect_prompt.return_value = True

        with (
            patch.object(self.processor.buffer, "is_timeout", return_value=True),
            patch.object(self.processor.buffer, "has_content", return_value=True),
        ):
            result = self.processor.check_timeout(0.1)

            assert result == ("test", True)
            assert self.processor.buffer.get_content() == ""

    def test_check_timeout_no_content(self):
        """Test timeout check when no content"""
        with patch.object(self.processor.buffer, "has_content", return_value=False):
            result = self.processor.check_timeout(0.1)
            assert result is None

    def test_get_last_prompt_for_mode_detection(self):
        """Test getting last prompt for mode detection"""
        self.processor.last_prompt_content = "A>"

        result = self.processor.get_last_prompt_for_mode_detection()

        assert result == "A>"
        assert self.processor.last_prompt_content == ""  # Should be cleared

    def test_get_last_prompt_for_mode_detection_empty(self):
        """Test getting last prompt when none available"""
        result = self.processor.get_last_prompt_for_mode_detection()
        assert result is None

    def test_check_prompt_candidate_instant_mode_success(self):
        """Test prompt candidate check in instant mode - success"""
        self.processor.set_instant_mode(True)
        self.processor.buffer.add_data("A>")
        self.mock_detector.detect_prompt.return_value = True

        with (
            patch.object(self.processor.buffer, "is_timeout", return_value=True),
            patch.object(self.processor.buffer, "has_content", return_value=True),
            patch.object(self.processor, "_is_likely_prompt", return_value=True),
        ):
            result = self.processor.check_prompt_candidate(0.02)

            assert result == ("A>", True)
            assert self.processor.buffer.get_content() == ""

    def test_check_prompt_candidate_instant_mode_not_likely(self):
        """Test prompt candidate check in instant mode - not likely prompt"""
        self.processor.set_instant_mode(True)
        self.processor.buffer.add_data("some text")

        with (
            patch.object(self.processor.buffer, "is_timeout", return_value=True),
            patch.object(self.processor.buffer, "has_content", return_value=True),
            patch.object(self.processor, "_is_likely_prompt", return_value=False),
        ):
            result = self.processor.check_prompt_candidate(0.02)

            assert result is None

    def test_check_prompt_candidate_instant_mode_no_timeout(self):
        """Test prompt candidate check in instant mode - no timeout"""
        self.processor.set_instant_mode(True)
        self.processor.buffer.add_data("A>")

        with (
            patch.object(self.processor.buffer, "is_timeout", return_value=False),
            patch.object(self.processor.buffer, "has_content", return_value=True),
        ):
            result = self.processor.check_prompt_candidate(0.02)

            assert result is None

    def test_check_prompt_candidate_success(self):
        """Test prompt candidate check success"""
        self.processor.buffer.add_data("A")
        self.mock_detector.is_prompt_candidate.return_value = True
        self.mock_detector.detect_prompt.return_value = True

        with (
            patch.object(self.processor.buffer, "is_timeout", return_value=True),
            patch.object(self.processor.buffer, "has_content", return_value=True),
        ):
            result = self.processor.check_prompt_candidate(0.02)

            assert result == ("A", True)
            assert self.processor.buffer.get_content() == ""

    def test_check_prompt_candidate_not_candidate(self):
        """Test prompt candidate check when not a candidate"""
        self.processor.buffer.add_data("test")
        self.mock_detector.is_prompt_candidate.return_value = False

        with patch.object(self.processor.buffer, "has_content", return_value=True):
            result = self.processor.check_prompt_candidate(0.02)
            assert result is None

    def test_check_prompt_candidate_no_timeout(self):
        """Test prompt candidate check when no timeout"""
        self.processor.buffer.add_data("A")
        self.mock_detector.is_prompt_candidate.return_value = True

        with (
            patch.object(self.processor.buffer, "is_timeout", return_value=False),
            patch.object(self.processor.buffer, "has_content", return_value=True),
        ):
            result = self.processor.check_prompt_candidate(0.02)

            assert result is None

    def test_split_prompt_data_with_prompt(self):
        """Test splitting data with prompt"""
        self.processor.buffer.add_data("line1\nline2\nA>")
        self.mock_detector.detect_prompt.side_effect = lambda x: x == "A>"

        result = self.processor._split_prompt_data()

        expected = [("line1", False), ("line2", False), ("A>", True)]
        assert result == expected

    def test_split_prompt_data_no_prompt(self):
        """Test splitting data without prompt"""
        self.processor.buffer.add_data("line1\nline2\ntext")
        self.mock_detector.detect_prompt.return_value = False

        result = self.processor._split_prompt_data()

        expected = [("line1", False), ("line2", False), ("text", False)]
        assert result == expected

    def test_split_prompt_data_empty_lines(self):
        """Test splitting data with empty lines"""
        self.processor.buffer.add_data("line1\n\nA>")
        self.mock_detector.detect_prompt.side_effect = lambda x: x == "A>"

        result = self.processor._split_prompt_data()

        expected = [("line1", False), ("A>", True)]
        assert result == expected

    def test_split_prompt_data_only_last_line(self):
        """Test splitting data with only last line"""
        self.processor.buffer.add_data("A>")
        self.mock_detector.detect_prompt.return_value = True

        result = self.processor._split_prompt_data()

        expected = [("A>", True)]
        assert result == expected

    def test_has_incomplete_data_true(self):
        """Test incomplete data detection - true case"""
        self.processor.buffer.add_data("incomplete")

        result = self.processor.has_incomplete_data()
        assert result is True

    def test_has_incomplete_data_false_with_newline(self):
        """Test incomplete data detection - false with newline"""
        self.processor.buffer.add_data("complete\n")

        result = self.processor.has_incomplete_data()
        assert result is False

    def test_has_incomplete_data_false_empty(self):
        """Test incomplete data detection - false when empty"""
        result = self.processor.has_incomplete_data()
        assert result is False


def test_process_echo_suppression_else_branch():
    from unittest.mock import Mock

    from msx_serial.core.data_processor import DataProcessor

    mock_detector = Mock()
    processor = DataProcessor(mock_detector)
    processor.set_last_command("ABC")
    processor.buffer.add_data("ABC")
    processor._process_echo_suppression("ABC")
    assert processor.buffer.get_content() == ""


def test_split_prompt_data_one_line_prompt():
    from unittest.mock import Mock

    from msx_serial.core.data_processor import DataProcessor

    mock_detector = Mock()
    mock_detector.detect_prompt.return_value = True
    processor = DataProcessor(mock_detector)
    processor.buffer.add_data("A>")
    result = processor._split_prompt_data()
    assert result == [("A>", True)]


def test_check_timeout_buffered_no_prompt():
    from unittest.mock import Mock, patch

    from msx_serial.core.data_processor import DataProcessor

    mock_detector = Mock()
    mock_detector.detect_prompt.return_value = False
    processor = DataProcessor(mock_detector)
    processor.buffer.add_data("test")
    with patch.object(processor.buffer, "is_timeout", return_value=True):
        result = processor.check_timeout()
    assert result == ("test", False)


def test_check_prompt_candidate_buffered_not_timeout():
    from unittest.mock import Mock, patch

    from msx_serial.core.data_processor import DataProcessor

    mock_detector = Mock()
    mock_detector.is_prompt_candidate.return_value = True
    mock_detector.detect_prompt.return_value = True
    processor = DataProcessor(mock_detector)
    processor.buffer.add_data("test")
    with patch.object(processor.buffer, "is_timeout", return_value=False):
        result = processor.check_prompt_candidate()
    assert result is None
